@@ -1,5 +1,5 @@
-//! Ipfs embed is a small, fast and reliable ipfs implementation designed for embedding in to
-//! complex p2p applications.
+//! Ipfs embed is a small, fast and reliable ipfs implementation designed for
+//! embedding in to complex p2p applications.
 //!
 //! ```no_run
 //! # #[async_std::main]
@@ -9,34 +9,6 @@
 //! ipfs.listen_on("/ip4/0.0.0.0/tcp/0".parse()?)?;
 //! # Ok(()) }
 //! ```
-use crate::db::StorageService;
-pub use crate::db::{StorageConfig, TempPin};
-pub use crate::net::{
-    generate_keypair, AddressRecord, AddressSource, BitswapConfig, BroadcastConfig, DnsConfig,
-    DocId, Event, GossipEvent, GossipsubConfig, Head, IdentifyConfig, KadConfig, Key, Keypair,
-    ListenerEvent, ListenerId, LocalStreamWriter, MdnsConfig, Multiaddr, NetworkConfig, PeerId,
-    PeerInfo, PeerRecord, PingConfig, PublicKey, Quorum, Record, SecretKey, SignedHead, StreamId,
-    StreamReader, SwarmEvents, SyncEvent, SyncQuery, ToLibp2p, TransportConfig,
-};
-use crate::net::{BitswapStore, NetworkService};
-#[cfg(feature = "telemetry")]
-pub use crate::telemetry::telemetry;
-use async_trait::async_trait;
-pub use db::Batch;
-use executor::Executor;
-use futures::stream::Stream;
-use libipld::codec::References;
-use libipld::error::BlockNotFound;
-pub use libipld::store::DefaultParams;
-use libipld::store::{Store, StoreParams};
-pub use libipld::{Block, Cid};
-use libipld::{Ipld, Result};
-use libp2p::kad::kbucket::Key as BucketKey;
-pub use libp2p::multiaddr;
-use prometheus::Registry;
-use std::collections::HashSet;
-use std::path::Path;
-use std::sync::Arc;
 
 mod db;
 mod executor;
@@ -46,6 +18,56 @@ mod telemetry;
 #[cfg(test)]
 mod test_util;
 
+/// convenience re-export of configuration types from libp2p
+pub mod config {
+    pub use libp2p::{
+        dns::{ResolverConfig, ResolverOpts},
+        gossipsub::GossipsubConfig,
+        identify::IdentifyConfig,
+        kad::record::store::MemoryStoreConfig as KadConfig,
+        mdns::MdnsConfig,
+        ping::PingConfig,
+    };
+    pub use libp2p_bitswap::BitswapConfig;
+    pub use libp2p_broadcast::BroadcastConfig;
+}
+
+#[cfg(feature = "telemetry")]
+pub use crate::telemetry::telemetry;
+pub use crate::{
+    db::{Batch, StorageConfig, StorageService, TempPin},
+    executor::Executor,
+    net::{
+        AddressSource, ConnectionFailure, Direction, DnsConfig, Event, GossipEvent, ListenerEvent,
+        NetworkConfig, PeerInfo, Rtt, SwarmEvents, SyncEvent, SyncQuery,
+    },
+};
+
+pub use libipld::{store::DefaultParams, Block, Cid};
+pub use libp2p::{
+    core::{connection::ListenerId, ConnectedPoint, Multiaddr, PeerId},
+    identity,
+    kad::{kbucket::Key as BucketKey, record::Key, PeerRecord, Quorum, Record},
+    multiaddr,
+    swarm::{AddressRecord, AddressScore},
+};
+
+use crate::net::NetworkService;
+use async_trait::async_trait;
+use chrono::{DateTime, Utc};
+use futures::stream::Stream;
+use libipld::{
+    codec::References,
+    error::BlockNotFound,
+    store::{Store, StoreParams},
+    Ipld, Result,
+};
+use libp2p::identity::ed25519::{Keypair, PublicKey};
+use libp2p_bitswap::BitswapStore;
+use parking_lot::Mutex;
+use prometheus::Registry;
+use std::{collections::HashSet, path::Path, sync::Arc, time::Duration};
+
 /// Ipfs configuration.
 #[derive(Debug)]
 pub struct Config {
@@ -56,19 +78,19 @@ pub struct Config {
 }
 
 impl Config {
-    /// Creates a default configuration from a `path` and a `cache_size`. If the `path` is `None`,
-    /// ipfs will use an in-memory block store.
+    /// Creates a default configuration from a `path` and a `cache_size`. If the
+    /// `path` is `None`, ipfs will use an in-memory block store.
     pub fn new(path: &Path, keypair: Keypair) -> Self {
         let sweep_interval = std::time::Duration::from_millis(10000);
-        let storage = StorageConfig::new(Some(path.join("blocks")), 0, sweep_interval);
-        let network = NetworkConfig::new(path.join("streams"), keypair);
+        let storage = StorageConfig::new(Some(path.join("blocks")), None, 0, sweep_interval);
+        let network = NetworkConfig::new(keypair);
         Self { storage, network }
     }
 }
 
 impl Default for Config {
     fn default() -> Self {
-        Self::new(Path::new("."), generate_keypair())
+        Self::new(Path::new("."), Keypair::generate())
     }
 }
 
@@ -102,7 +124,7 @@ where
     }
 
     fn insert(&mut self, block: &Block<P>) -> Result<()> {
-        self.0.insert(block)
+        self.0.insert(block.clone())
     }
 
     fn missing_blocks(&mut self, cid: &Cid) -> Result<Vec<Cid>> {
@@ -116,8 +138,8 @@ where
 {
     /// Creates a new `Ipfs` from a `Config`.
     ///
-    /// This starts three background tasks. The swarm, garbage collector and the dht cleanup
-    /// tasks run in the background.
+    /// This starts three background tasks. The swarm, garbage collector and the
+    /// dht cleanup tasks run in the background.
     pub async fn new(config: Config) -> Result<Self> {
         let executor = Executor::new();
         Self::new0(config, executor).await
@@ -174,6 +196,12 @@ where
         self.network.remove_address(peer, addr)
     }
 
+    /// Removes all unconnected peers without addresses which have been
+    /// in this state for at least the given duration
+    pub fn prune_peers(&self, min_age: Duration) {
+        self.network.prune_peers(min_age);
+    }
+
     /// Dials a `PeerId` using a known address.
     pub fn dial(&self, peer: &PeerId) {
         self.network.dial(peer);
@@ -181,8 +209,7 @@ where
 
     /// Dials a `PeerId` using `Multiaddr`.
     pub fn dial_address(&self, peer: &PeerId, addr: Multiaddr) {
-        self.network.add_address(peer, addr);
-        self.network.dial(peer);
+        self.network.dial_address(peer, addr);
     }
 
     /// Bans a `PeerId` from the swarm, dropping all existing connections and
@@ -202,7 +229,7 @@ where
     }
 
     /// Returns a list of connected peers.
-    pub fn connections(&self) -> Vec<(PeerId, Multiaddr)> {
+    pub fn connections(&self) -> Vec<(PeerId, Multiaddr, DateTime<Utc>, Direction)> {
         self.network.connections()
     }
 
@@ -216,8 +243,8 @@ where
         self.network.peer_info(peer)
     }
 
-    /// Bootstraps the dht using a set of bootstrap nodes. After bootstrap completes it
-    /// provides all blocks in the block store.
+    /// Bootstraps the dht using a set of bootstrap nodes. After bootstrap
+    /// completes it provides all blocks in the block store.
     pub async fn bootstrap(&self, nodes: &[(PeerId, Multiaddr)]) -> Result<()> {
         self.network.bootstrap(nodes).await?;
         Ok(())
@@ -228,7 +255,8 @@ where
         self.network.is_bootstrapped()
     }
 
-    /// Gets the closest peer to a key. Useful for finding the `Multiaddr` of a `PeerId`.
+    /// Gets the closest peer to a key. Useful for finding the `Multiaddr` of a
+    /// `PeerId`.
     pub async fn get_closest_peers<K>(&self, key: K) -> Result<()>
     where
         K: Into<BucketKey<K>> + Into<Vec<u8>> + Clone,
@@ -253,7 +281,7 @@ where
     }
 
     /// Gets a record from the dht.
-    pub async fn get_record(&self, key: &Key, quorum: Quorum) -> Result<Vec<PeerRecord>> {
+    pub async fn get_record(&self, key: Key, quorum: Quorum) -> Result<Vec<PeerRecord>> {
         self.network.get_record(key, quorum).await
     }
 
@@ -267,30 +295,32 @@ where
         self.network.remove_record(key)
     }
 
-    /// Subscribes to a `topic` returning a `Stream` of messages. If all `Stream`s for
-    /// a topic are dropped it unsubscribes from the `topic`.
+    /// Subscribes to a `topic` returning a `Stream` of messages. If all
+    /// `Stream`s for a topic are dropped it unsubscribes from the `topic`.
     pub fn subscribe(&self, topic: &str) -> Result<impl Stream<Item = GossipEvent>> {
         self.network.subscribe(topic)
     }
 
-    /// Publishes a new message in a `topic`, sending the message to all subscribed peers.
+    /// Publishes a new message in a `topic`, sending the message to all
+    /// subscribed peers.
     pub fn publish(&self, topic: &str, msg: Vec<u8>) -> Result<()> {
         self.network.publish(topic, msg)
     }
 
-    /// Publishes a new message in a `topic`, sending the message to all subscribed connected peers.
+    /// Publishes a new message in a `topic`, sending the message to all
+    /// subscribed connected peers.
     pub fn broadcast(&self, topic: &str, msg: Vec<u8>) -> Result<()> {
         self.network.broadcast(topic, msg)
     }
 
-    /// Creates a temporary pin in the block store. A temporary pin is not persisted to disk
-    /// and is released once it is dropped.
+    /// Creates a temporary pin in the block store. A temporary pin is not
+    /// persisted to disk and is released once it is dropped.
     pub fn create_temp_pin(&self) -> Result<TempPin> {
         self.storage.create_temp_pin()
     }
 
     /// Adds a new root to a temporary pin.
-    pub fn temp_pin(&self, tmp: &TempPin, cid: &Cid) -> Result<()> {
+    pub fn temp_pin(&self, tmp: &mut TempPin, cid: &Cid) -> Result<()> {
         self.storage.temp_pin(tmp, std::iter::once(*cid))
     }
 
@@ -314,8 +344,8 @@ where
         }
     }
 
-    /// Either returns a block if it's in the block store or tries to retrieve it from
-    /// a peer.
+    /// Either returns a block if it's in the block store or tries to retrieve
+    /// it from a peer.
     pub async fn fetch(&self, cid: &Cid, providers: Vec<PeerId>) -> Result<Block<P>> {
         if let Some(data) = self.storage.get(cid)? {
             let block = Block::new_unchecked(*cid, data);
@@ -333,20 +363,21 @@ where
     }
 
     /// Inserts a block in to the block store.
-    pub fn insert(&self, block: &Block<P>) -> Result<()> {
+    pub fn insert(&self, block: Block<P>) -> Result<()> {
         self.storage.insert(block)?;
         Ok(())
     }
 
-    /// Manually runs garbage collection to completion. This is mainly useful for testing and
-    /// administrative interfaces. During normal operation, the garbage collector automatically
-    /// runs in the background.
+    /// Manually runs garbage collection to completion. This is mainly useful
+    /// for testing and administrative interfaces. During normal operation,
+    /// the garbage collector automatically runs in the background.
     pub async fn evict(&self) -> Result<()> {
         self.storage.evict().await
     }
 
     pub fn sync(&self, cid: &Cid, providers: Vec<PeerId>) -> SyncQuery<P> {
         let missing = self.storage.missing_blocks(cid).ok().unwrap_or_default();
+        tracing::trace!(cid = %cid, missing = %missing.len(), "sync");
         self.network.sync(*cid, providers, missing)
     }
 
@@ -355,36 +386,34 @@ where
         self.storage.alias(alias.as_ref(), cid)
     }
 
+    /// List all known aliases.
+    pub fn aliases(&self) -> Result<Vec<(Vec<u8>, Cid)>> {
+        self.storage.aliases()
+    }
+
     /// Returns the root of an alias.
     pub fn resolve<T: AsRef<[u8]> + Send + Sync>(&self, alias: T) -> Result<Option<Cid>> {
         self.storage.resolve(alias.as_ref())
     }
 
-    /// Returns a list of aliases preventing a `Cid` from being garbage collected.
-    pub fn reverse_alias(&self, cid: &Cid) -> Result<Option<Vec<Vec<u8>>>> {
+    /// Returns a list of aliases preventing a `Cid` from being garbage
+    /// collected.
+    pub fn reverse_alias(&self, cid: &Cid) -> Result<Option<HashSet<Vec<u8>>>> {
         self.storage.reverse_alias(cid)
     }
 
-    /// Flushes the block store. After `flush` completes successfully it is guaranteed that
-    /// all writes have been persisted to disk.
+    /// Flushes the block store. After `flush` completes successfully it is
+    /// guaranteed that all writes have been persisted to disk.
     pub async fn flush(&self) -> Result<()> {
         self.storage.flush().await
     }
 
-    /// Perform a set of storage operations in a batch, and discards the result.
+    /// Perform a set of storage operations in a batch
     ///
-    /// You should keep a batch open for as short as possible.
-    /// You *must not* create a batch inside a batch, or use the outer ipfs.
-    pub fn read_batch<F: FnOnce(&mut Batch<'_, P>) -> Result<R>, R>(&self, f: F) -> Result<R> {
-        self.storage.ro("read_batch", f)
-    }
-
-    /// Perform a set of storage operations in a batch, and stores the result.
-    ///
-    /// You should keep a batch open for as short as possible.
-    /// You *must not* create a batch inside a batch, or use the outer ipfs.
-    pub fn write_batch<R>(&self, f: impl FnOnce(&mut Batch<'_, P>) -> Result<R>) -> Result<R> {
-        self.storage.rw("write_batch", f)
+    /// The batching concerns only the CacheTracker, it implies no atomicity
+    /// guarantees!
+    pub fn batch_ops<R>(&self, f: impl FnOnce(&mut Batch<'_, P>) -> Result<R>) -> Result<R> {
+        self.storage.rw("batch_ops", f)
     }
 
     /// Registers prometheus metrics in a registry.
@@ -398,56 +427,6 @@ where
     pub fn swarm_events(&self) -> SwarmEvents {
         self.network.swarm_events()
     }
-
-    /// Returns the documents both local and replicated.
-    pub fn docs(&self) -> Result<Vec<DocId>> {
-        self.network.docs()
-    }
-
-    /// Returns the streams both local and replicated.
-    pub fn streams(&self) -> Result<Vec<StreamId>> {
-        self.network.streams()
-    }
-
-    /// Returns the streams both local and replicated for the given document id.
-    pub fn substreams(&self, doc: DocId) -> Result<Vec<StreamId>> {
-        self.network.substreams(doc)
-    }
-
-    /// Adds the peers to a replicated stream.
-    pub fn stream_add_peers(&self, doc: DocId, peers: impl Iterator<Item = PeerId>) {
-        self.network.stream_add_peers(doc, peers)
-    }
-
-    /// Returns the current head of a stream.
-    pub fn stream_head(&self, id: &StreamId) -> Result<Option<SignedHead>> {
-        self.network.stream_head(id)
-    }
-
-    /// Returns a reader for the slice.
-    pub fn stream_slice(&self, id: &StreamId, start: u64, len: u64) -> Result<StreamReader> {
-        self.network.stream_slice(id, start, len)
-    }
-
-    /// Removes a local or replicated stream.
-    pub fn stream_remove(&self, id: &StreamId) -> Result<()> {
-        self.network.stream_remove(id)
-    }
-
-    /// Returns a writter to append to a local stream.
-    pub fn stream_append(&self, id: DocId) -> Result<LocalStreamWriter> {
-        self.network.stream_append(id)
-    }
-
-    /// Subscribes to a replicated stream.
-    pub fn stream_subscribe(&self, id: &StreamId) -> Result<()> {
-        self.network.stream_subscribe(id)
-    }
-
-    /// Updates the head of a replicated stream.
-    pub fn stream_update_head(&self, head: SignedHead) {
-        self.network.stream_update_head(head)
-    }
 }
 
 #[async_trait]
@@ -456,14 +435,14 @@ where
     Ipld: References<P::Codecs>,
 {
     type Params = P;
-    type TempPin = Arc<TempPin>;
+    type TempPin = Arc<Mutex<TempPin>>;
 
     fn create_temp_pin(&self) -> Result<Self::TempPin> {
-        Ok(Arc::new(Ipfs::create_temp_pin(self)?))
+        Ok(Arc::new(Mutex::new(Ipfs::create_temp_pin(self)?)))
     }
 
     fn temp_pin(&self, tmp: &Self::TempPin, cid: &Cid) -> Result<()> {
-        Ipfs::temp_pin(self, tmp, cid)
+        Ipfs::temp_pin(self, &mut *tmp.lock(), cid)
     }
 
     fn contains(&self, cid: &Cid) -> Result<bool> {
@@ -475,7 +454,7 @@ where
     }
 
     fn insert(&self, block: &Block<P>) -> Result<()> {
-        let _ = Ipfs::insert(self, block)?;
+        let _ = Ipfs::insert(self, block.clone())?;
         Ok(())
     }
 
@@ -488,7 +467,7 @@ where
     }
 
     fn reverse_alias(&self, cid: &Cid) -> Result<Option<Vec<Vec<u8>>>> {
-        Ipfs::reverse_alias(self, cid)
+        Ipfs::reverse_alias(self, cid).map(|x| x.map(|x| x.into_iter().collect()))
     }
 
     async fn flush(&self) -> Result<()> {
@@ -508,13 +487,10 @@ where
 mod tests {
     use super::*;
     use async_std::future::timeout;
-    use futures::join;
-    use futures::stream::StreamExt;
-    use libipld::cbor::DagCborCodec;
-    use libipld::multihash::Code;
-    use libipld::raw::RawCodec;
-    use libipld::store::DefaultParams;
-    use libipld::{alias, ipld};
+    use futures::{join, stream::StreamExt};
+    use libipld::{
+        alias, cbor::DagCborCodec, ipld, multihash::Code, raw::RawCodec, store::DefaultParams,
+    };
     use std::time::Duration;
     use tempdir::TempDir;
 
@@ -528,9 +504,9 @@ mod tests {
     async fn create_store(enable_mdns: bool) -> Result<(Ipfs<DefaultParams>, TempDir)> {
         let tmp = TempDir::new("ipfs-embed")?;
         let sweep_interval = Duration::from_millis(10000);
-        let storage = StorageConfig::new(None, 10, sweep_interval);
+        let storage = StorageConfig::new(None, None, 10, sweep_interval);
 
-        let mut network = NetworkConfig::new(tmp.path().into(), generate_keypair());
+        let mut network = NetworkConfig::new(Keypair::generate());
         if !enable_mdns {
             network.mdns = None;
         }
@@ -552,9 +528,9 @@ mod tests {
         tracing_try_init();
         let (store, _tmp) = create_store(false).await?;
         let block = create_block(b"test_local_store")?;
-        let tmp = store.create_temp_pin()?;
-        store.temp_pin(&tmp, block.cid())?;
-        let _ = store.insert(&block)?;
+        let mut tmp = store.create_temp_pin()?;
+        store.temp_pin(&mut tmp, block.cid())?;
+        let _ = store.insert(block.clone())?;
         let block2 = store.get(block.cid())?;
         assert_eq!(block.data(), block2.data());
         Ok(())
@@ -567,12 +543,12 @@ mod tests {
         let (store1, _tmp) = create_store(true).await?;
         let (store2, _tmp) = create_store(true).await?;
         let block = create_block(b"test_exchange_mdns")?;
-        let tmp1 = store1.create_temp_pin()?;
-        store1.temp_pin(&tmp1, block.cid())?;
-        let _ = store1.insert(&block)?;
+        let mut tmp1 = store1.create_temp_pin()?;
+        store1.temp_pin(&mut tmp1, block.cid())?;
+        let _ = store1.insert(block.clone())?;
         store1.flush().await?;
-        let tmp2 = store2.create_temp_pin()?;
-        store2.temp_pin(&tmp2, block.cid())?;
+        let mut tmp2 = store2.create_temp_pin()?;
+        store2.temp_pin(&mut tmp2, block.cid())?;
         let block2 = store2
             .fetch(block.cid(), vec![store1.local_peer_id()])
             .await?;
@@ -600,14 +576,14 @@ mod tests {
 
         let block = create_block(b"test_exchange_kad")?;
         let key = Key::new(&block.cid().to_bytes());
-        let tmp1 = store1.create_temp_pin()?;
-        store1.temp_pin(&tmp1, block.cid())?;
-        store1.insert(&block)?;
+        let mut tmp1 = store1.create_temp_pin()?;
+        store1.temp_pin(&mut tmp1, block.cid())?;
+        store1.insert(block.clone())?;
         store1.provide(key.clone()).await?;
         store1.flush().await?;
 
-        let tmp2 = store2.create_temp_pin()?;
-        store2.temp_pin(&tmp2, block.cid())?;
+        let mut tmp2 = store2.create_temp_pin()?;
+        store2.temp_pin(&mut tmp2, block.cid())?;
         let providers = store2.providers(key).await?;
         let block2 = store2
             .fetch(block.cid(), providers.into_iter().collect())
@@ -676,9 +652,9 @@ mod tests {
         let c2 = create_ipld_block(&ipld!({ "c": [a1.cid(), b2.cid()] }))?;
         let x = alias!(x);
 
-        let _ = local1.insert(&a1)?;
-        let _ = local1.insert(&b1)?;
-        let _ = local1.insert(&c1)?;
+        let _ = local1.insert(a1.clone())?;
+        let _ = local1.insert(b1.clone())?;
+        let _ = local1.insert(c1.clone())?;
         local1.alias(x, Some(c1.cid()))?;
         local1.flush().await?;
         assert_pinned!(&local1, &a1);
@@ -692,8 +668,8 @@ mod tests {
         assert_pinned!(&local2, &b1);
         assert_pinned!(&local2, &c1);
 
-        let _ = local2.insert(&b2)?;
-        let _ = local2.insert(&c2)?;
+        let _ = local2.insert(b2.clone())?;
+        let _ = local2.insert(c2.clone())?;
         local2.alias(x, Some(c2.cid()))?;
         local2.flush().await?;
         assert_pinned!(&local2, &a1);
@@ -758,7 +734,7 @@ mod tests {
                 Quorum::One,
             )
             .await?;
-        let records = stores[1].0.get_record(&key, Quorum::One).await?;
+        let records = stores[1].0.get_record(key, Quorum::One).await?;
         assert_eq!(records.len(), 1);
         Ok(())
     }
@@ -794,7 +770,7 @@ mod tests {
 
         stores[0]
             .0
-            .publish(&topic, b"hello gossip".to_vec())
+            .publish(topic, b"hello gossip".to_vec())
             .unwrap();
 
         for (idx, subscription) in subscriptions.iter_mut().enumerate() {
@@ -838,7 +814,7 @@ mod tests {
         // Check broadcast subscription
         stores[0]
             .0
-            .broadcast(&topic, b"hello broadcast".to_vec())
+            .broadcast(topic, b"hello broadcast".to_vec())
             .unwrap();
 
         for subscription in &mut subscriptions[1..] {
@@ -853,7 +829,7 @@ mod tests {
         // trigger cleanup
         stores[0]
             .0
-            .broadcast(&topic, b"r u still listening?".to_vec())
+            .broadcast(topic, b"r u still listening?".to_vec())
             .unwrap();
 
         let mut last_sub = subscriptions.drain(..1).next().unwrap();
@@ -883,16 +859,14 @@ mod tests {
     #[async_std::test]
     async fn test_batch_read() -> Result<()> {
         tracing_try_init();
-        let tmp = TempDir::new("ipfs-embed")?;
-        let network = NetworkConfig::new(tmp.path().into(), generate_keypair());
-        let storage = StorageConfig::new(None, 1000000, Duration::from_secs(3600));
+        let network = NetworkConfig::new(Keypair::generate());
+        let storage = StorageConfig::new(None, None, 1000000, Duration::from_secs(3600));
         let ipfs = Ipfs::<DefaultParams>::new(Config { storage, network }).await?;
         let a = create_block(b"a")?;
         let b = create_block(b"b")?;
-        ipfs.insert(&a)?;
-        ipfs.insert(&b)?;
-        let has_blocks =
-            ipfs.read_batch(|db| Ok(db.contains(a.cid())? && db.contains(b.cid())?))?;
+        ipfs.insert(a.clone())?;
+        ipfs.insert(b.clone())?;
+        let has_blocks = ipfs.batch_ops(|db| Ok(db.contains(a.cid())? && db.contains(b.cid())?))?;
         assert!(has_blocks);
         Ok(())
     }
@@ -900,26 +874,26 @@ mod tests {
     #[async_std::test]
     async fn test_batch_write() -> Result<()> {
         tracing_try_init();
-        let tmp = TempDir::new("ipfs-embed")?;
-        let network = NetworkConfig::new(tmp.path().into(), generate_keypair());
-        let storage = StorageConfig::new(None, 1000000, Duration::from_secs(3600));
+        let network = NetworkConfig::new(Keypair::generate());
+        let storage = StorageConfig::new(None, None, 1000000, Duration::from_secs(3600));
         let ipfs = Ipfs::<DefaultParams>::new(Config { storage, network }).await?;
         let a = create_block(b"a")?;
         let b = create_block(b"b")?;
         let c = create_block(b"c")?;
         let d = create_block(b"d")?;
-        ipfs.write_batch(|db| {
-            db.insert(&a)?;
-            db.insert(&b)?;
+        ipfs.batch_ops(|db| {
+            db.insert(a.clone())?;
+            db.insert(b.clone())?;
             Ok(())
         })?;
         assert!(ipfs.contains(a.cid())? && ipfs.contains(b.cid())?);
-        let _: anyhow::Result<()> = ipfs.write_batch(|db| {
-            db.insert(&c)?;
-            db.insert(&d)?;
+        #[allow(unreachable_code)]
+        let _: anyhow::Result<()> = ipfs.batch_ops(|db| {
+            db.insert(c.clone())?;
             anyhow::bail!("nope!");
+            db.insert(d.clone())?;
         });
-        assert!(!ipfs.contains(c.cid())? && ipfs.contains(b.cid())?);
+        assert!(!ipfs.contains(d.cid())? && ipfs.contains(c.cid())? && ipfs.contains(b.cid())?);
         Ok(())
     }
 
@@ -939,7 +913,7 @@ mod tests {
         let size: usize = blocks.iter().map(|block| block.data().len()).sum();
         tracing::info!("chain built {} blocks, {} bytes", blocks.len(), size);
         for block in blocks.iter() {
-            let _ = a.insert(block)?;
+            let _ = a.insert(block.clone())?;
         }
         a.flush().await?;
 
@@ -979,7 +953,7 @@ mod tests {
         let size: usize = blocks.iter().map(|block| block.data().len()).sum();
         tracing::info!("chain built {} blocks, {} bytes", blocks.len(), size);
         for block in blocks.iter() {
-            let _ = a.insert(block)?;
+            let _ = a.insert(block.clone())?;
         }
         a.flush().await?;
 
